@@ -1,5 +1,4 @@
-# ABSTRACT: Cache, time consuming subroutines or paid api calls
-#
+# ABSTRACT: Cache time consuming subroutines or paid api calls
 # George Bouras , george.mpouras@yandex.com
 # Hellas/Athens , 02 Feb 2019
 
@@ -11,27 +10,28 @@ use warnings;
 
 
 use File::Path::Tiny;
-our $VERSION = '2.0.6';
+our $VERSION = '2.1.0';
 our @ISA     = ();
 our $ERROR   = 0;
 
 
 #	Object constructor
-#	Properties starting from __ are module's privates
-#	my $cache = Cache->new(cache_dir=>'/cache/records/app01', callback=>'Get_new_data', expire_sec=>3, verbose=>'False') or die "oups $Cache::ERROR\n";
+#	__* properties are module privates
+#	my $cache = Cache->new(cache_dir=>'/tmp', callback=>'Get_new_data', expire_sec=>3, verbose=>'False') or die "oups $Cache::ERROR\n";
 sub new
 {
-my $class = shift || __PACKAGE__;
-my $self  ={
-error			=> 0,
-error_message	=> 'ok',
-cache_dir		=> $^O=~/(?i)MSWin/ ? (local $_="$ENV{TEMP}\\cache", s/\\/\//g, $_) : '/tmp/cache',
-callback		=> '__NEW_RECORD',	# Subroutine name that cache new data
-expire_sec		=> 3600,			# After how many seconds the record will be considered expired and a new one should cached using the callback
-verbose			=> 'False',			# Verbose if TRUE or 1
-__userclass		=> $class,			# Class of callback
-__record		=> undef,
-__tmp			=> undef};
+my $class	= shift || __PACKAGE__;
+my $self	={
+error		=> 0,
+error_msg	=> 'ok',
+expire_sec	=> 3600,			# After how many seconds the record will be considered expired and a new one should cached using the callback
+verbose		=> 'False',			# Verbose if TRUE or 1
+callback	=> '__NEW_RECORD',	# Subroutine name that cache new data
+cache_dir	=> $^O=~/(?i)MSWin/ ? (local $_="$ENV{TEMP}\\cache", s/\\/\//g, $_) : '/tmp/cache',
+__userclass	=> $class,			# Class of callback
+__dir		=> undef,			# The subdirectory under the "cache_dir" of your cached files
+__record	=> undef,
+__tmp		=> undef};
 
 # Define properties from the arguments. At @_ remain only the args that are not matching to a propery
 
@@ -41,7 +41,7 @@ __tmp			=> undef};
 
 			if ('__' eq substr $_[$i],0,2) {
 			$ERROR = "You tried to set the internal private property \"$_[$i]\" to value \"$_[$j]\" , sorry this is not permitted. Valid user properties are : ". join(', ', sort grep /^[^_]/, keys %{$self});
-			@{$self}{qw/error error_message/}=(10,$ERROR);
+			@{$self}{qw/error error_msg/}=(10,$ERROR);
 			return undef
 			}
 
@@ -49,15 +49,13 @@ __tmp			=> undef};
 		}
 		else {
 		$ERROR = "You tried to define the invalid property \"$_[$i]\" to value \"$_[$j]\" , sorry this is not permitted . Valid proerties are : ". join(', ', sort grep /^[^_]/ , keys %{$self});
-		@{$self}{qw/error error_message/}=(11,$ERROR);
+		@{$self}{qw/error error_msg/}=(11,$ERROR);
 		return undef
 		}
 	}
 
-$self->{verbose} = $self->{verbose}=~/(?i)t|y|1/ ? 1 : 0;
-
 # Check if the subroutine exists	$self->{callback}  
-# You can call as					$self->{__userclass}->can($self->{callback})->( 1547328808 )
+# You can call it as				$self->{__userclass}->can($self->{callback})->( 1547328808 )
 
 	if ($self->{callback} ne '__NEW_RECORD') {
 	$self->{__userclass} = [caller]->[0]
@@ -65,53 +63,57 @@ $self->{verbose} = $self->{verbose}=~/(?i)t|y|1/ ? 1 : 0;
 
 	if ('CODE' ne ref $self->{__userclass}->can($self->{callback}))	{
 	$ERROR = "Function \"$self->{__userclass}::$self->{callback}\" does not exist";
-	@{$self}{qw/error error_message/}=(12,$ERROR);
+	@{$self}{qw/error error_msg/}=(12,$ERROR);
 	return undef
 	}
 
+$self->{verbose}	= $self->{verbose}=~/(?i)t|y|1/ ?1:0;
+$self->{cache_dir}	=~s/[\/\\]*$//;
+$self->{__dir}		= "$self->{cache_dir}/$self->{__userclass}-$self->{callback}";
+
 # Create and clear the top cache directory if missing
 
-	if (-d $self->{cache_dir}) {
+	if (-d $self->{__dir}) {
 
-		if (-f "$self->{cache_dir}/lock" ) {
+		if (-f "$self->{__dir}/lock" ) {
 
 			if ( $^O=~/(?i)MSWin/ ) {
-			print "Removing lock file : $self->{cache_dir}/lock\n" if $self->{verbose};
+			print "Removing lock file : $self->{__dir}/lock\n" if $self->{verbose};
 
-				unless ( unlink "$self->{cache_dir}/lock" ) {
-				$ERROR = "Could not remove lock \"$self->{cache_dir}/lock\" because \"$!\"";
-				@{$self}{qw/error error_message/}=(13,$ERROR);
+				unless ( unlink "$self->{__dir}/lock" ) {
+				$ERROR = "Could not remove lock \"$self->{__dir}/lock\" because \"$!\"";
+				@{$self}{qw/error error_msg/}=(13,$ERROR);
 				return undef
 				}
 			}
 			else {
 
-				if (open __LOCK, '<', "$self->{cache_dir}/lock") {
+				if (open __LOCK, '<', "$self->{__dir}/lock") {
 				$self->{__tmp} = readline __LOCK;
 				close __LOCK;
 
 					unless (-d "/proc/$self->{__tmp}") {
-					print "Removing lock file \"$self->{cache_dir}/lock\" of non existing process \"$self->{__tmp}\"\n" if $self->{verbose};
+					print "Removing lock file \"$self->{__dir}/lock\" of non existing process \"$self->{__tmp}\"\n" if $self->{verbose};
 
-						unless ( unlink "$self->{cache_dir}/lock" ) {
-						$ERROR = "Could not remove lock \"$self->{cache_dir}/lock\" of non existing process \"$self->{__tmp}\" because \"$!\"";
-						@{$self}{qw/error error_message/}=(14,$ERROR);
+						unless ( unlink "$self->{__dir}/lock" ) {
+						$ERROR = "Could not remove lock \"$self->{__dir}/lock\" of non existing process \"$self->{__tmp}\" because \"$!\"";
+						@{$self}{qw/error error_msg/}=(14,$ERROR);
 						return undef
 						}							
 					}
 				}
 				else {
-				$ERROR = "Could not read lock file \"$self->{cache_dir}/lock\" because \"$!\"\n";
-				@{$self}{qw/error error_message/}=(15,$ERROR);
+				$ERROR = "Could not read lock file \"$self->{__dir}/lock\" because \"$!\"\n";
+				@{$self}{qw/error error_msg/}=(15,$ERROR);
 				return undef
 				}
 			}
 		}
 
-	unless (opendir CACHE, $self->{cache_dir}) {$ERROR="Could not read cache directory \"$self->{cache_dir}\" because \"$!\""; @{$self}{qw/error error_message/}=(16,$ERROR); return undef}
+	unless (opendir CACHE, $self->{__dir}) {$ERROR="Could not read cache directory \"$self->{__dir}\" because \"$!\""; @{$self}{qw/error error_msg/}=(16,$ERROR); return undef}
 
 		while (my $node = readdir CACHE) {
-			if ((-d "$self->{cache_dir}/$node") && ($node =~/^\d+$/)) {
+			if ((-d "$self->{__dir}/$node") && ($node =~/^\d+$/)) {
 			$self->{__record} = $node;
 			last
 			}
@@ -121,9 +123,9 @@ $self->{verbose} = $self->{verbose}=~/(?i)t|y|1/ ? 1 : 0;
 	}
 	else {
 
-		unless ( File::Path::Tiny::mk $self->{cache_dir} ) {
-		$ERROR = "Could not create the top cache directory \"$self->{cache_dir}\" because \"$!\"";
-		@{$self}{qw/error error_message/}=(17,$ERROR);
+		unless ( File::Path::Tiny::mk $self->{__dir} ) {
+		$ERROR = "Could not create the top cache directory \"$self->{__dir}\" because \"$!\"";
+		@{$self}{qw/error error_msg/}=(17,$ERROR);
 		return undef
 		}
 	}
@@ -180,54 +182,54 @@ close FILE
 #
 sub __CACHE_DATA
 {
-my $obj = ((exists $_[0]) && (__PACKAGE__ eq ref $_[0])) ? shift : __NOT_AN_OBJECT([caller 0]->[3], @_);
+my $obj = ((exists $_[0]) && (__PACKAGE__ eq ref $_[0])) ? shift : __NOT_AN_OBJECT([caller 0]->[3],@_);
 my $time= time;
 
-	unless (-d "$obj->{cache_dir}/$time") {
+	unless (-d "$obj->{__dir}/$time") {
 
-		if (-f "$obj->{cache_dir}/lock") {
+		if (-f "$obj->{__dir}/lock") {
 
-			if (open __LOCK, '<', "$obj->{cache_dir}/lock") {
+			if (open __LOCK, '<', "$obj->{__dir}/lock") {
 			$obj->{__tmp} = readline __LOCK;
 			close __LOCK;
 			$ERROR = "An other process \"$obj->{__tmp}\" is trying to get new cache data write now\n";
-			@{$obj}{qw/error error_message/}=(100,$ERROR);
+			@{$obj}{qw/error error_msg/}=(2000,$ERROR);
 			return undef
 			}
 			else {
-			$ERROR = "Could not read lock file \"$obj->{cache_dir}/lock\" because \"$!\"\n";
-			@{$obj}{qw/error error_message/}=(99,$ERROR);
+			$ERROR = "Could not read lock file \"$obj->{__dir}/lock\" because \"$!\"\n";
+			@{$obj}{qw/error error_msg/}=(20,$ERROR);
 			return undef
 			}
 		}
 		else {
 
-			# create lock file
-			if (open __LOCK, '>', "$obj->{cache_dir}/lock") {
+			# Create lock file
+			if (open __LOCK, '>', "$obj->{__dir}/lock") {
 			print __LOCK $$;
 			close __LOCK
 			}
 			else {
-			$ERROR = "Could not create lock file: $obj->{cache_dir}/lock\n";
-			@{$obj}{qw/error error_message/}=(98,$ERROR);
+			$ERROR = "Could not create lock file: $obj->{__dir}/lock\n";
+			@{$obj}{qw/error error_msg/}=(21,$ERROR);
 			return undef
 			}
 		}
 
-		# create cache record subdirectory
-		unless (mkdir "$obj->{cache_dir}/$time") {
-		$ERROR = "Could not create cache record directory : $obj->{cache_dir}/$time\n";
-		@{$obj}{qw/error error_message/}=(97,$ERROR);
+		# Create cache record subdirectory
+		unless (mkdir "$obj->{__dir}/$time") {
+		$ERROR = "Could not create cache record directory : $obj->{__dir}/$time\n";
+		@{$obj}{qw/error error_msg/}=(22,$ERROR);
 		return undef
 		}
 
-		unless ( $obj->{__userclass}->can($obj->{callback})->("$obj->{cache_dir}/$time", @_) ) {
+		unless ( $obj->{__userclass}->can($obj->{callback})->("$obj->{__dir}/$time", @_) ) {
 		$ERROR = "Cache new data fuction $obj->{__userclass}::$obj->{callback} return a false value";
-		@{$obj}{qw/error error_message/}=(95,$ERROR);		
+		@{$obj}{qw/error error_msg/}=(23,$ERROR);		
 		return undef
 		}
 
-	unlink "$obj->{cache_dir}/lock"
+	unlink "$obj->{__dir}/lock"
 	}
 
 $time
@@ -238,33 +240,33 @@ $time
 #	Returns the cache record. If it is expired or not exists get new data using the __CACHE_DATA
 sub get
 {
-my $obj = ((exists $_[0]) && (__PACKAGE__ eq ref $_[0])) ? shift : __NOT_AN_OBJECT([caller 0]->[3], @_);
+my $obj = ((exists $_[0]) && (__PACKAGE__ eq ref $_[0])) ? shift : __NOT_AN_OBJECT([caller 0]->[3],@_);
 	
 	if (defined $obj->{__record}) {
 
 		if ($obj->{expire_sec} > time - $obj->{__record}) {
 		print 'use existing '.(time - $obj->{__record}) ."/$obj->{expire_sec}\n" if $obj->{verbose};
-		"$obj->{cache_dir}/$obj->{__record}"
+		"$obj->{__dir}/$obj->{__record}"
 		}
 		else {
-		print 'New get '.(time - $obj->{__record})."/$obj->{expire_sec}\n" if $obj->{verbose};
+		print "New get\n" if $obj->{verbose};
 		$obj->{__tmp} = $obj->__CACHE_DATA(@_);
 
 			if ($obj->{__tmp}) {
 
-				unless ( File::Path::Tiny::rm "$obj->{cache_dir}/$obj->{__record}" ) {
-				$ERROR = "Could not remove expired cache directory \"$obj->{cache_dir}/$obj->{__record}\" because \"$!\"";
-				@{$obj}{qw/error error_message/}=(1,$ERROR);
+				unless (File::Path::Tiny::rm "$obj->{__dir}/$obj->{__record}") {
+				$ERROR = "Could not remove expired cache directory \"$obj->{__dir}/$obj->{__record}\" because \"$!\"";
+				@{$obj}{qw/error error_msg/}=(30,$ERROR);
 				return undef
 				}
 
 			$obj->{__record} = $obj->{__tmp};
-			"$obj->{cache_dir}/$obj->{__record}"
+			"$obj->{__dir}/$obj->{__record}"
 			}
 			else {
 
-				if ($obj->{error}==100) {
-				"$obj->{cache_dir}/$obj->{__record}"
+				if ($obj->{error}==2000) {
+				"$obj->{__dir}/$obj->{__record}"
 				}
 				else {
 				undef
@@ -277,7 +279,7 @@ my $obj = ((exists $_[0]) && (__PACKAGE__ eq ref $_[0])) ? shift : __NOT_AN_OBJE
 	$obj->{__record} = $obj->__CACHE_DATA(@_);
 
 		if ($obj->{__record}) {
-		"$obj->{cache_dir}/$obj->{__record}"
+		"$obj->{__dir}/$obj->{__record}"
 		}
 		else {
 		undef
@@ -285,6 +287,7 @@ my $obj = ((exists $_[0]) && (__PACKAGE__ eq ref $_[0])) ? shift : __NOT_AN_OBJE
 	}
 }
 
+
 1;
 
 __END__
@@ -295,11 +298,11 @@ __END__
 
 =head1 NAME
 
-SimpleDir - Cache, time consuming subroutines or paid api calls
+SimpleDir - Cache time consuming subroutines or paid api calls
 
 =head1 VERSION
 
-version 2.0.6
+version 2.1.0
 
 =head1 SYNOPSIS
 
@@ -307,12 +310,12 @@ version 2.0.6
   use Cache::SimpleDir;
 
   my $cache  =  SimpleDir->new(
-  cache_dir  => '/tmp/cache/key1',
+  cache_dir  => '/tmp',
   callback   => 'GetWeather',
   expire_sec => 1800,
   verbose    => 'false') or die $SimpleDir::ERROR;
 
-  my $where_are_my_data = $cache->get('a','b','c') or die "oups $SimpleDir::ERROR\n";
+  my $where_are_my_data = $cache->get('a','b','c') or die $SimpleDir::ERROR;
   print "data are at: $where_are_my_data\n";
 
   #     How to get and cache new data
@@ -339,14 +342,16 @@ Write at the I<callback> subroutine the code, that generate new data.
 Its first argument is always the directory that you should write your cached files.
 Any optional argument used at the B<get> is passed at the I<callback>
 
+It is thread safe.
+
 =head1 ABSTRACT
 
-Cache, time consuming subroutines or paid api calls. It is thread safe, and can be used from different processes using the same I<cache_dir>
+Cache time consuming subroutines or paid api calls
 
 =head1 ERROR HANDLING
 
 On error B<get> returns FALSE. Sets the error message at the variable $SimpleDir::ERROR
-and at the property $obj->error_message while the error code is at $obj->error
+and at the property $obj->error_msg while the error code is at $obj->error
 
 =head1 METHODS
 
@@ -362,6 +367,14 @@ B<expire_sec> I<After how many seconds the record will be considered expired and
 
 B<verbose>    I<Verbose operation if TRUE or 1>
 
+There is not support for multiple cache keys at the same object.
+This is by design, because it must be fast and simple.
+If you want multiple keys, then create multiple objects with different properties e.g.
+
+  my $key1 = SimpleDir->new(callback=>'Sub1', cache_dir=>'/tmp', expire_sec=>300);
+  my $key2 = SimpleDir->new(callback=>'Sub2', cache_dir=>'/tmp', expire_sec=>800);
+  ...
+
 =head2 get
 
 Returns the cache directory where your files/dirs are stored.
